/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: DemoConfig.java,v 1.5 2009/08/19 16:50:58 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.config;

import com.unitesk.testfusion.core.config.*;
import com.unitesk.testfusion.core.config.template.*;
import com.unitesk.testfusion.core.dependency.*;
import com.unitesk.testfusion.core.model.*;
import com.unitesk.testfusion.core.situation.EmptySituation;
import com.unitesk.testfusion.demo.model.*;
import com.unitesk.testfusion.demo.model.isa.*;
import com.unitesk.testfusion.demo.model.isa.arithmetic.*;
import com.unitesk.testfusion.demo.model.isa.arithmetic.situation.*;
import com.unitesk.testfusion.demo.model.isa.branch.*;
import com.unitesk.testfusion.demo.model.isa.branch.situation.*;
import com.unitesk.testfusion.demo.model.isa.logic.*;
import com.unitesk.testfusion.demo.model.isa.memory.*;
import com.unitesk.testfusion.demo.model.isa.system.*;
import com.unitesk.testfusion.gui.GUI;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class DemoConfig extends TestConfig
{
    public static String BEQ_TRACE_ARRAY = "beq_trace_array";

    public static int MAX_TRACE_LENGTH  = 3;
    public static int MIN_BRANCH_NUMBER = 2;
    public static int MAX_BRANCH_NUMBER = 2;
    public static int MIN_TEMPLATE_SIZE = 3;
    public static int MAX_TEMPLATE_SIZE = 3;
    
    protected DemoProcessor processor = new DemoProcessor();
    
    public DemoConfig()
    {
        // Processor instruction set
        ProcessorConfig architecture = new ProcessorConfig(processor);
        
        ///////////////////////////////////////////////////////////////////////////////////////////
        // NOP
        ///////////////////////////////////////////////////////////////////////////////////////////
        
        GroupConfig nopGroup = new GroupConfig("nop");

        // NOP instruction
        InstructionConfig nopInstruction = new InstructionConfig(new NOPInstruction());
        nopInstruction.registerSituation(new SituationConfig(new EmptySituation()));
        nopInstruction.setEquivalenceClass("Empty Instruction");
        nopGroup.registerInstruction(nopInstruction);

        architecture.registerGroup(nopGroup);

        ///////////////////////////////////////////////////////////////////////////////////////////
        // Logic arithmetic
        ///////////////////////////////////////////////////////////////////////////////////////////

        GroupConfig logicGroup = new GroupConfig("logic");

        // LA pseudo-instruction
        InstructionConfig laInstruction = new InstructionConfig(new LAInstruction());
        logicGroup.registerInstruction(laInstruction);
        
        // LUI instruction
        InstructionConfig luiInstruction = new InstructionConfig(new LUIInstruction());
        logicGroup.registerInstruction(luiInstruction);

        // SUB instruction
        InstructionConfig oriInstruction = new InstructionConfig(new ORIInstruction());
        logicGroup.registerInstruction(oriInstruction);
        
        architecture.registerGroup(logicGroup);
        
        ///////////////////////////////////////////////////////////////////////////////////////////
        // Integer arithmetic
        ///////////////////////////////////////////////////////////////////////////////////////////

        GroupConfig integerGroup = new GroupConfig("integer");

        // ADD instruction
        InstructionConfig addInstruction = new InstructionConfig(new ADDIInstruction());
        addInstruction.registerSituation(new SituationConfig(new ADDINormalSituation()));
        addInstruction.registerSituation(new SituationConfig(new ADDIOverflowSituation()));
        addInstruction.setEquivalenceClass("Integer Arithmetic");
        integerGroup.registerInstruction(addInstruction);

        architecture.registerGroup(integerGroup);
        
        ///////////////////////////////////////////////////////////////////////////////////////////
        // Branching
        ///////////////////////////////////////////////////////////////////////////////////////////
        GroupConfig branchGroup = new GroupConfig("branch");

        // BEQ instruction
        InstructionConfig beqInstruction = new InstructionConfig(new BEQInstruction());
        beqInstruction.registerSituation(new SituationConfig(new BEQTraceSituation(BEQ_TRACE_ARRAY)));
        beqInstruction.setEquivalenceClass("Conditional Jump");
        branchGroup.registerInstruction(beqInstruction);
        
        // J instruction
        InstructionConfig jInstruction = new InstructionConfig(new JInstruction());
        jInstruction.registerSituation(new SituationConfig(new JTraceSituation()));
        jInstruction.setEquivalenceClass("Unconditional Jump");
        branchGroup.registerInstruction(jInstruction);

        architecture.registerGroup(branchGroup);
        
        ///////////////////////////////////////////////////////////////////////////////////////////
        // Memory
        ///////////////////////////////////////////////////////////////////////////////////////////
        GroupConfig memoryGroup = new GroupConfig("memory");

        // LOAD instruction
        InstructionConfig loadInstruction  = new InstructionConfig(new LWInstruction());
        memoryGroup.registerInstruction(loadInstruction);

        // STORE instruction
        InstructionConfig storeInstruction = new InstructionConfig(new SWInstruction());
        memoryGroup.registerInstruction(storeInstruction);
        
        architecture.registerGroup(memoryGroup);

        ///////////////////////////////////////////////////////////////////////////////////////////
        // System
        ///////////////////////////////////////////////////////////////////////////////////////////
        GroupConfig systemGroup = new GroupConfig("system");

        // Wait
        InstructionConfig waitInstruction = new InstructionConfig(new WAITInstruction());
        systemGroup.registerInstruction(waitInstruction);
        
        architecture.registerGroup(systemGroup);
        
        setProcessor(architecture);
        
        // Dependencies.
        DependencyListConfig dependencies = new DependencyListConfig();

        RegisterDependencyConfig dependency = new RegisterDependencyConfig(new RegisterDependency(processor, OperandType.REGISTER));
        
        dependencies.registerDependency(dependency);
        
        setDependencies(dependencies);
        
        // Section.
        SectionConfig section = createSection("template");
        
        architecture = section.getProcessor();
        
        nopGroup     = architecture.getGroup("nop");
        integerGroup = architecture.getGroup("integer");
        branchGroup  = architecture.getGroup("branch");
        
        nopGroup.setSelectedWithPropagation();
        integerGroup.setSelectedWithPropagation();
        branchGroup.setSelectedWithPropagation();
        
        OptionsConfig options = section.getOptions();
        
        options.setIndex(OptionsConfig.BRANCH_TEMPLATE_ITERATOR);

        BranchTemplateIteratorConfig iterator = (BranchTemplateIteratorConfig)options.getTemplateIterator(OptionsConfig.BRANCH_TEMPLATE_ITERATOR);

        iterator.setMaxBranchExecution(MAX_TRACE_LENGTH);
        iterator.setMinBranchNumber(MIN_BRANCH_NUMBER);
        iterator.setMaxBranchNumber(MAX_BRANCH_NUMBER);
        iterator.setMinTemplateSize(MIN_TEMPLATE_SIZE);
        iterator.setMaxTemplateSize(MAX_TEMPLATE_SIZE);
        
        registerSection(section);
        
        // Generator.
        setTestSize(200);
        setGenerator(new DemoGenerator(200));        
    }
    
    public static void main(String[] args)
    {  
       GUI.createAndShowGUI(new DemoConfig());
    }
}
