/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: DemoTemplate.java,v 1.9 2009/08/18 14:52:39 vorobyev Exp $
 */

package com.unitesk.testfusion.demo.model.config;

import com.unitesk.testfusion.core.context.GeneratorContext;
import com.unitesk.testfusion.core.engine.DefaultTestProgramTemplate;
import com.unitesk.testfusion.core.generator.Random;
import com.unitesk.testfusion.core.model.Program;
import com.unitesk.testfusion.core.model.PseudoInstruction;
import com.unitesk.testfusion.demo.model.DemoProcessor;
import com.unitesk.testfusion.demo.model.isa.NOPInstruction;
import com.unitesk.testfusion.demo.model.isa.branch.JInstruction;
import com.unitesk.testfusion.demo.model.isa.logic.ORIInstruction;
import com.unitesk.testfusion.demo.model.isa.system.WAITInstruction;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class DemoTemplate extends DefaultTestProgramTemplate
{
    protected static Program declareWordArray(String arrayName, int arraySize, boolean zero)
    {
        Program program = new Program();
        
        program.append(new PseudoInstruction(arrayName + ":"));
        
        for(int i = 0; i < arraySize; i++)
        {
            int value = zero ? 0x0 : Random.int32();
            
            program.append(new PseudoInstruction(".word " + "0x" + Integer.toHexString(value)));
        }
        
        return program;
    }
    
    protected DemoProcessor processor;
    
    protected String traceArray;
    
    protected int maxBranchNumber;
    protected int traceArraySize;
    
    public DemoTemplate(DemoProcessor processor, int maxBranchNumber, String traceArray, int traceArraySize)
    {
        this.processor       = processor;
        this.maxBranchNumber = maxBranchNumber;
        this.traceArray      = traceArray;
        this.traceArraySize  = traceArraySize;
    }

    public int getMaxBranchNumber()
    {
        return maxBranchNumber;
    }
    
    public void setMaxBranchNumber(int maxBranchNumber)
    {
        this.maxBranchNumber = maxBranchNumber;
    }
    
    @Override
    public Program getTestPrefix(GeneratorContext context)
    {
        Program program = new Program();

        program.append(new PseudoInstruction(".nolist") );
        program.append(new PseudoInstruction(".set noreorder") );
        program.append(new PseudoInstruction(".set noat") );
        
        program.append(new PseudoInstruction() );
        program.append(new PseudoInstruction(".list") );
        program.append(new PseudoInstruction(".text") );
        program.append(new PseudoInstruction(".globl __start") );
        program.append(new PseudoInstruction() );
        program.append(new PseudoInstruction(".org 0x2000"));

        program.append(new PseudoInstruction());
        program.append(new PseudoInstruction("__start:"));
        program.append(new JInstruction("test"));
        program.append(new NOPInstruction());

        // Error end
        program.append(new PseudoInstruction());
        program.append(new PseudoInstruction(singleLineComment() + " Erroneous termination"));
        program.append(new PseudoInstruction("error_end:"));
        program.append(new ORIInstruction(processor.getRegister(1), processor.getRegister(0), 0xdead));
        program.append(new NOPInstruction());
        program.append(new WAITInstruction(0x7ffff));

        // Success end
        program.append(new PseudoInstruction());
        program.append(new PseudoInstruction(singleLineComment() + " Successful termination"));
        program.append(new PseudoInstruction("success_end:"));
        program.append(new ORIInstruction(processor.getRegister(1), processor.getRegister(0), 0x0));
        program.append(new NOPInstruction());
        program.append(new WAITInstruction(0x7ffff));

        program.append(new PseudoInstruction());
        program.append(new PseudoInstruction("test:"));
        program.append(new PseudoInstruction());

        return program;
    }

    @Override
    public Program getTestSituationPrefix(GeneratorContext context, boolean first, boolean last)
    {
        Program program = new Program();

        int action = context.getTestActionNumber();
        
        program.append(new JInstruction(getTestSituationLabel(action)));
        program.append(new NOPInstruction());
        
        program.append(new PseudoInstruction(singleLineComment() + " Label for backward jumps"));
        program.append(new PseudoInstruction(getBackwardLabel(action) + ":"));
        program.append(new JInstruction(getTestSituationLabel(action + 1) ));
        program.append(new NOPInstruction());
        
        program.append(new PseudoInstruction());
        
        return program;
    }
    
    @Override
    public Program getTestActionSuffix(GeneratorContext context, boolean first, boolean last)
    {
        Program program = new Program();
        
        int action = context.getTestActionNumber();
        
        program.append(new PseudoInstruction(singleLineComment() + " Label for forward jumps"));
        program.append(new PseudoInstruction(getForwardLabel(action) + ":"));
        
        if(last)
            { program.append(new PseudoInstruction(getTestSituationLabel(action + 1) + ":")); }
        
        program.append(new PseudoInstruction());
        
        return program;
    }
    
    @Override
    public Program getTestSuffix(GeneratorContext context)
    {
        Program program = new Program();
        
        program.append(new JInstruction("success_end"));
        program.append(new NOPInstruction());
        
        for(int i = 0; i < maxBranchNumber; i++)
        {
            program.append(new PseudoInstruction());
            program.append(declareWordArray(traceArray + "_" + i, traceArraySize, true));
        }
        
        program.append(new PseudoInstruction());
        
        return program;
    }
    
    @Override
    public String getForwardLabel(int n)
    {
        return "1";
    }
    
    @Override
    public String getBackwardLabel(int n)
    {
        return "0";
    }
}
