/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: ADDIOverflowSituation.java,v 1.2 2009/08/13 15:54:29 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.arithmetic.situation;

import com.unitesk.testfusion.core.context.GeneratorContext;
import com.unitesk.testfusion.core.generator.IntegerArithmetic;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.demo.model.isa.DemoSituation;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class ADDIOverflowSituation extends DemoSituation
{
    public ADDIOverflowSituation()
    {
        super("Overflow", "Produces integer overflow exception for ADDI instruction");
    }
    
    protected ADDIOverflowSituation(ADDIOverflowSituation r)
    {
        super(r);
    }

    @Override
    public boolean construct(Processor processor, GeneratorContext context)
    {
        Operand rsOperand = getOperand("rs");
        Operand imOperand = getOperand("im");
        
        boolean rsDefined = context.isDefinedRegister(rsOperand.getRegister());
        
        IntegerArithmetic.lhs32 = rsOperand.getIntegerValue();
        
        if(!IntegerArithmetic.add32x16_overflow(rsDefined, false))
            { return false; }
        
        rsOperand.setIntegerValue(IntegerArithmetic.lhs32);
        imOperand.setShortValue(IntegerArithmetic.rhs16);
        
        return true;
    }
    
    @Override
    public ADDIOverflowSituation clone()
    {
        return new ADDIOverflowSituation(this);
    }
}
