/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: BEQInstruction.java,v 1.3 2009/08/13 15:54:31 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.branch;

import com.unitesk.testfusion.core.exception.UnsatisfiedCondition;
import com.unitesk.testfusion.core.model.ContentType;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.OperandType;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.core.model.register.Register;
import com.unitesk.testfusion.core.util.Utils;
import com.unitesk.testfusion.demo.model.DemoRegister;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class BEQInstruction extends BranchInstruction
{
    public BEQInstruction()
    {
        super("beq");
        
        registerOperand(new Operand(this, "rs",     Operand.INPUT,  OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "rt",     Operand.INPUT,  OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "offset", Operand.INPUT,  OperandType.IMMEDIATE, ContentType.OFFSET));
    }
    
    public BEQInstruction(DemoRegister rs, DemoRegister rt, short offset)
    {
        this();
        
        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        
        Operand offsetOperand = getOperand("offset");
        
        rsOperand.setRegister(rs);
        rtOperand.setRegister(rt);
        
        offsetOperand.setShortValue(offset);
    }
    
    public BEQInstruction(DemoRegister rs, DemoRegister rt, String label)
    {
        this();
        
        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        
        rsOperand.setRegister(rs);
        rtOperand.setRegister(rt);
        
        this.label = label;
    }

    @Override
    public boolean isConditionalBranch()
    {
        return true;
    }
    
    @Override
    public void calculate(Processor processor) 
    {
        super.calculate(processor);

        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        
        if(rsOperand.getLongValue() != rtOperand.getLongValue())
            { setException(new UnsatisfiedCondition()); }
    }        
    
    @Override
    public void execute(Processor processor) 
    {
        super.execute(processor);
        
        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        
        Register rsRegister = rsOperand.getRegister();
        Register rtRegister = rtOperand.getRegister();
        
        if(rsRegister.getLongValue() != rtRegister.getLongValue())
            { setException(new UnsatisfiedCondition()); }
    }

    @Override
    public String toString()
    {
        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        
        Operand offsetOperand = getOperand("offset");
        
        return "beq " + rsOperand.getRegister() + ", " + rtOperand.getRegister() + ", " +
            (label != null ? label : "0x" + Utils.toHexString(offsetOperand.getShortValue()));
    }
    
    @Override
    public BEQInstruction createInstruction()
    {
        return new BEQInstruction();
    }
}
