/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: BEQTraceSituation.java,v 1.2 2009/08/19 16:51:00 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.branch.situation;

import com.unitesk.testfusion.core.context.GeneratorContext;
import com.unitesk.testfusion.core.generator.Random;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.core.model.Program;
import com.unitesk.testfusion.core.template.iterator.branch.BranchExecution;
import com.unitesk.testfusion.demo.model.DemoProcessor;
import com.unitesk.testfusion.demo.model.DemoRegister;
import com.unitesk.testfusion.demo.model.isa.arithmetic.ADDIInstruction;
import com.unitesk.testfusion.demo.model.lib.LoadImmediate32Program;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class BEQTraceSituation extends DemoTraceSituation
{
    public BEQTraceSituation(String arrayPrefix)
    {
        super("BEQ Trace", "Produces different execution traces for BEQ instruction", arrayPrefix);
    }

    protected BEQTraceSituation(BEQTraceSituation r)
    {
        super(r);
    }

    @Override
    public void satisfyCondition()
    {
        Operand rsOperand = getOperand("rs");
        
        rsOperand.setLongValue(0);
    }
    
    @Override
    public void violateCondition()
    {
        Operand rsOperand = getOperand("rs");
        
        rsOperand.setLongValue(Random.int32_non_equal(0));
    }

    @Override
    public boolean canUseSimpleStep()
    {
        return trueNumber == 1;
    }
    
    @Override
    public Program simpleStep()
    {
        return new Program(new ADDIInstruction(rsRegister, rsRegister, (short)-1));
    }

    @Override
    public Program simplePrepare(Processor processor, GeneratorContext context, int layer)
    {
        int i, size;
        
        Program program = new Program();
        Operand rsOperand = getOperand("rs");
        
        int rtValue = 0;
        int rsValue = rtValue;
        
        size = branchTrace.size();
        for(i = 0; i < size; i++)
        {
            BranchExecution execution = branchTrace.get(i);

            rsValue += execution.getCoverageCount(canInsertStepIntoBlock());

            if(execution.condition())
                { break; }
        }
        
        program.append(new LoadImmediate32Program((DemoProcessor)processor, (DemoRegister)rsOperand.getRegister(), rsValue));
        
        return program;
    }

    @Override
    public BEQTraceSituation clone()
    {
        return new BEQTraceSituation(this);
    }
}
