/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: DemoTraceSituation.java,v 1.4 2009/08/19 16:51:00 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.branch.situation;

import com.unitesk.testfusion.core.context.GeneratorContext;
import com.unitesk.testfusion.core.model.Instruction;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.OperandType;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.core.model.Program;
import com.unitesk.testfusion.core.template.iterator.branch.BranchExecution;
import com.unitesk.testfusion.core.template.iterator.branch.SimpleTraceSituation;
import com.unitesk.testfusion.demo.model.DemoProcessor;
import com.unitesk.testfusion.demo.model.DemoRegister;
import com.unitesk.testfusion.demo.model.isa.NOPInstruction;
import com.unitesk.testfusion.demo.model.isa.arithmetic.ADDIInstruction;
import com.unitesk.testfusion.demo.model.isa.branch.BranchInstruction;
import com.unitesk.testfusion.demo.model.isa.logic.LAInstruction;
import com.unitesk.testfusion.demo.model.isa.memory.LWInstruction;
import com.unitesk.testfusion.demo.model.isa.memory.SWInstruction;
import com.unitesk.testfusion.demo.model.lib.LoadImmediate32Program;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public abstract class DemoTraceSituation extends SimpleTraceSituation
{
	protected DemoRegister rsRegister;
	protected DemoRegister rtRegister;
	protected DemoRegister baseRegister;

	protected String arrayPrefix;
	
	protected boolean condition;
	
	protected int changeNumber;
	protected int trueNumber;
	protected int falseNumber;
	protected int changePosition;
	
	public DemoTraceSituation(String name, String text, String arrayPrefix)
	{
        super(name, text);
        
		this.arrayPrefix = arrayPrefix;
	}

	protected DemoTraceSituation(DemoTraceSituation r)
	{
		super(r);
		
		rsRegister     = r.rsRegister;
		rtRegister     = r.rtRegister;
		baseRegister   = r.baseRegister;
		arrayPrefix    = r.arrayPrefix;
		condition      = r.condition;
		changeNumber   = r.changeNumber;
		trueNumber     = r.trueNumber;
		falseNumber    = r.falseNumber;
		changePosition = r.changePosition;
	}

	public String getConditionArrayName()
	{
		return arrayPrefix + "_" + branchNumber;
	}
	
	@Override
    public void init(Processor processor, GeneratorContext context)
    {
        DemoProcessor demoProcessor = (DemoProcessor)processor;
        
        if(!branchTrace.isEmpty())
        {
        	condition      = branchTrace.getCondition(0);
	    	changeNumber   = branchTrace.getChangeNumber();
        	trueNumber     = branchTrace.getTrueNumber();
        	falseNumber    = branchTrace.getFalseNumber();
	    	changePosition = branchTrace.getChangePosition();
        }

    	rtRegister   = (DemoRegister)demoProcessor.getRegister(0);
    	rsRegister   = (DemoRegister)context.getRegister(OperandType.REGISTER);
    	baseRegister = (DemoRegister)context.getRegister(OperandType.REGISTER);

        context.useRegister(rtRegister);
        context.useRegister(rsRegister);
        context.useRegister(baseRegister);
    }
    
    @Override
    public void useRegisters(Processor processor, GeneratorContext context)
    {
        context.useRegister(rtRegister);
    	context.useRegister(rsRegister);
    	context.useRegister(baseRegister);
    }
	
	@Override
	public Program generalStep()
	{
		Program program = new Program();
		
		program.append(new LWInstruction(rsRegister, (short)0x0, baseRegister));
		program.append(new ADDIInstruction(baseRegister, baseRegister, (short)0x4));

		return program;
	}

    @Override
    public boolean construct(Processor processor, GeneratorContext context)
    {
    	BranchInstruction instruction = (BranchInstruction)getInstruction();
    	
    	instruction.setLabel(getBranchLabel() + (isBackwardBranch() ? "b" : "f"));
    	
    	Operand rsOperand = getOperand("rs");
    	Operand rtOperand = getOperand("rt");
    	
    	if(rsOperand != null)
    		{ rsOperand.setRegister(rsRegister); }
    	
    	if(rtOperand != null)
    		{ rtOperand.setRegister(rtRegister); }
        
        return true;
    }

	@Override
	public Program basicPrepare(Processor processor, GeneratorContext context, int layer)
	{
		Program program = new Program();
    	Operand rsOperand = getOperand("rs");
    	
    	satisfyCondition(condition);
    	program.append(new LoadImmediate32Program((DemoProcessor)processor, (DemoRegister)rsOperand.getRegister(), rsOperand.getIntegerValue()));
    	
    	return program;
	}

	@Override
	public Program generalPrepare(Processor processor, GeneratorContext context, int layer)
	{
		int i, j, size1, size2;
		
		Program program = new Program();
		Operand rsOperand = getOperand("rs");
		
        boolean makeInitialization = true;
        DemoRegister temp = (DemoRegister)context.getRegister(OperandType.REGISTER);
        
        program.append(new LAInstruction(baseRegister, getConditionArrayName()));
        
        size1 = branchTrace.size();
        for(i = 0; i < size1; i++)
        {
        	BranchExecution execution = branchTrace.get(i);
        	
        	if(i == 0 && execution.getCoverageCount(canInsertStepIntoBlock()) > 0)
        		{ makeInitialization = false; }
        	
        	satisfyCondition(execution.condition());
    		
        	size2 = execution.getCoverageCount(canInsertStepIntoBlock());
        	for(j = 0; j < size2; j++)
        	{
        		program.append(new LoadImmediate32Program((DemoProcessor)processor, temp, rsOperand.getIntegerValue()));
        		program.append(new SWInstruction(temp, (short)0x0, baseRegister));
        		program.append(new ADDIInstruction(baseRegister, baseRegister, (short)4));
        	}
        }
        
        program.append(new LAInstruction(baseRegister, getConditionArrayName()));
        
        if(makeInitialization)
        	{ program.append(basicPrepare(processor, context, layer)); }
        
        return program;
	}
    
	@Override
    public boolean isHazard(Program program, int position)
    {
        if(position == 0)
            { return false; }
        
        Instruction instruction = program.getInstruction(position - 1);
        
        return instruction.isBranchInstruction();
    }
    
    @Override
    public Program resolveHazard(Program program, int position)
    {
        return new Program(new NOPInstruction());
    }
}
