/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: LUIInstruction.java,v 1.2 2009/08/13 15:54:33 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.logic;

import com.unitesk.testfusion.core.model.ContentType;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.OperandType;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.core.model.register.Register;
import com.unitesk.testfusion.core.util.Utils;
import com.unitesk.testfusion.demo.model.DemoRegister;
import com.unitesk.testfusion.demo.model.isa.DemoInstruction;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class LUIInstruction extends DemoInstruction
{
    public LUIInstruction()
    {
        super("lui");
        
        registerOperand(new Operand(this, "rt", Operand.OUTPUT, OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "im", Operand.INPUT,  OperandType.IMMEDIATE, ContentType.HALF_WORD));
    }

    public LUIInstruction(DemoRegister rt, int immediate)
    {
        this();
        
        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
        
        rtOperand.setRegister(rt);
        imOperand.setIntegerValue(immediate);
    }

    @Override
    public void calculate(Processor processor) 
    {
        super.calculate(processor);

        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
        
        rtOperand.setLongValue((imOperand.getIntegerValue() & 0xffff) << 16);
    }        
    
    @Override
    public void execute(Processor processor) 
    {
        super.execute(processor);

        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
        
        Register rtRegister = rtOperand.getRegister();
        
        rtRegister.setLongValue((imOperand.getIntegerValue() & 0xffff) << 16);
    }

    @Override
    public String toString()
    {
        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
       
        return "lui " + rtOperand.getRegister() +  ", 0x" + Utils.toHexString(imOperand.getShortValue());
    }

    @Override
    public LUIInstruction createInstruction()
    {
        return new LUIInstruction();
    }
}
