/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: ORIInstruction.java,v 1.2 2009/08/13 15:54:33 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.logic;

import com.unitesk.testfusion.core.model.ContentType;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.OperandType;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.core.model.register.Register;
import com.unitesk.testfusion.core.util.Utils;
import com.unitesk.testfusion.demo.model.DemoRegister;
import com.unitesk.testfusion.demo.model.isa.DemoInstruction;


/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class ORIInstruction extends DemoInstruction
{
    public ORIInstruction()
    {
        super("ori");
        
        registerOperand(new Operand(this, "rt", Operand.OUTPUT, OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "rs", Operand.INPUT,  OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "im", Operand.INPUT,  OperandType.IMMEDIATE, ContentType.HALF_WORD));
    }

    public ORIInstruction(DemoRegister rt, DemoRegister rs, int im)
    {
        this();
        
        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
        
        rsOperand.setRegister(rs);
        rtOperand.setRegister(rt);
        imOperand.setIntegerValue(im);
    }

    @Override
    public void calculate(Processor processor) 
    {
        super.calculate(processor);

        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
        
        rtOperand.setLongValue(rsOperand.getLongValue() | (imOperand.getIntegerValue() & 0xffffffff));
    }        
    
    @Override
    public void execute(Processor processor) 
    {
        super.execute(processor);

        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
        
        Register rtRegister = rtOperand.getRegister();
        Register rsRegister = rsOperand.getRegister();
        
        rtRegister.setLongValue(rsRegister.getLongValue() | (imOperand.getIntegerValue() & 0xffffffff));
    }
    
    @Override
    public String toString()
    {
        Operand rsOperand = getOperand("rs");
        Operand rtOperand = getOperand("rt");
        Operand imOperand = getOperand("im");
       
        return "ori " + rtOperand.getRegister() + ", " + rsOperand.getRegister() +  ", 0x" + Utils.toHexString(imOperand.getShortValue());
    }
    
    @Override
    public ORIInstruction createInstruction()
    {
        return new ORIInstruction();
    }
    
}
