/* 
 * Copyright (c) 2009 ISPRAS
 *
 * Institute for System Programming of Russian Academy of Sciences
 * 25 B.Kommunisticheskaya st. Moscow 109004 Russia
 *
 * All rights reserved.
 * 
 * $Id: MemoryInstruction.java,v 1.1 2009/07/14 12:37:09 kamkin Exp $
 */

package com.unitesk.testfusion.demo.model.isa.memory;

import com.unitesk.testfusion.core.model.ContentType;
import com.unitesk.testfusion.core.model.Operand;
import com.unitesk.testfusion.core.model.OperandType;
import com.unitesk.testfusion.core.model.Processor;
import com.unitesk.testfusion.demo.model.DemoRegister;
import com.unitesk.testfusion.demo.model.exception.AddressError;
import com.unitesk.testfusion.demo.model.isa.DemoInstruction;

/**
 * @author <a href="emailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public abstract class MemoryInstruction extends DemoInstruction
{
    protected boolean isLoad;
    
    protected MemoryInstruction(String name, boolean isLoad)
    {
        super(name);
        
        int INPUT_OR_OUTPUT = isLoad ? Operand.OUTPUT : Operand.INPUT;
        
        this.isLoad = isLoad;
        
        registerOperand(new Operand(this, "rt",     INPUT_OR_OUTPUT, OperandType.REGISTER,  ContentType.WORD));
        registerOperand(new Operand(this, "offset", Operand.INPUT,   OperandType.IMMEDIATE, ContentType.OFFSET));
        registerOperand(new Operand(this, "base",   Operand.INPUT,   OperandType.REGISTER,  ContentType.DATA_ADDRESS));
        
        registerException(new AddressError());
    }
    
    protected MemoryInstruction(String name, boolean isLoad, DemoRegister rt, short offset, DemoRegister base)
    {
        this(name, isLoad);
        
        Operand rtOperand = getOperand("rt");
        Operand offsetOperand = getOperand("offset");
        Operand baseOperand = getOperand("base");
        
        rtOperand.setRegister(rt);
        offsetOperand.setShortValue(offset);
        baseOperand.setRegister(base);
    }
    
    public boolean isLoad()
    {
        return isLoad;
    }
    
    @Override
    public void calculate(Processor processor)
    {
        super.calculate(processor);
        
        // Memory model is absent.
    }
    
    @Override
    public void execute(Processor processor)
    {
        super.execute(processor);
        
        // Memory model is absent.
    }
    
    @Override
    public String toString()
    {
        Operand rt = getOperand("rt");
        Operand base = getOperand("base");
        Operand offset = getOperand("offset"); 
        
        return getName() + " " + rt.getRegister() + ", " + offset.getShortValue() + "(" + base.getRegister() + ")";
    }    
}
